﻿using DisplayS57File;
using System;
using System.Collections.ObjectModel;
using System.Data;
using System.Drawing;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;


public partial class Sample : Form
{
    private MapEngine mapEngine;
    S57FeatureLayer featureLayer;
    Bitmap bitmap;

    public Sample()
    {
        InitializeComponent();
    }

    private void DisplayASimpleMap_Load(object sender, EventArgs e)
    {
        bitmap = new Bitmap(map.Width, map.Height);
        mapEngine = new MapEngine();

        featureLayer = new S57FeatureLayer(@"../data/US1HA02M/US1HA02M.000");
        featureLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(S57StyleFactory.GetMaritimeStyle());
        featureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
        mapEngine.StaticLayers.Add(featureLayer);

        mapEngine.CurrentExtent = ExtentHelper.GetDrawingExtent(new RectangleShape(-162.94238506875, 24.0831275, -154.81250225625, 20.204953671875), map.Width, map.Height);
        DrawImage();
    }

    private void DrawImage()
    {
        bitmap = new Bitmap(map.Width, map.Height);
        mapEngine.OpenAllLayers();
        mapEngine.DrawStaticLayers(bitmap, GeographyUnit.DecimalDegree);
        mapEngine.CloseAllLayers();

        map.Image = bitmap;
    }

    private void ToolBar_ButtonClick(object sender, ToolBarButtonClickEventArgs e)
    {
        switch (e.Button.Tag.ToString())
        {
            case "Zoom In":
                mapEngine.CurrentExtent.ScaleDown(50);
                break;
            case "Zoom Out":
                mapEngine.CurrentExtent.ScaleUp(50);
                break;
            case "Full Extent":
                mapEngine.CurrentExtent = ExtentHelper.GetDrawingExtent(new RectangleShape(-133.291207591435, 52.4457064436166, -65.7348915320567, 18.5637124044684), map.Width, map.Height);
                break;
            case "Pan Left":
                mapEngine.CurrentExtent = ExtentHelper.Pan(mapEngine.CurrentExtent, PanDirection.Left, 20);
                break;
            case "Pan Right":
                mapEngine.CurrentExtent = ExtentHelper.Pan(mapEngine.CurrentExtent, PanDirection.Right, 20);
                break;
            case "Pan Up":
                mapEngine.CurrentExtent = ExtentHelper.Pan(mapEngine.CurrentExtent, PanDirection.Up, 20);
                break;
            case "Pan Down":
                mapEngine.CurrentExtent = ExtentHelper.Pan(mapEngine.CurrentExtent, PanDirection.Down, 20);
                break;
            default:
                break;
        }
        DrawImage();
    }

    private void map_ClientSizeChanged(object sender, EventArgs e)
    {
        if (bitmap != null && (map.Width > bitmap.Width || map.Height > bitmap.Height))
        {
            bitmap = new Bitmap(map.Width, map.Height);
            double scale = new ZoomLevelSet().GetZoomLevel(mapEngine.CurrentExtent, map.Width, GeographyUnit.DecimalDegree).Scale;
            mapEngine.CurrentExtent = ExtentHelper.SnapToZoomLevel((RectangleShape)mapEngine.CurrentExtent.CloneDeep(), GeographyUnit.DecimalDegree, map.Width, map.Height, new ZoomLevelSet());
            mapEngine.CurrentExtent = ExtentHelper.ZoomToScale(scale, mapEngine.CurrentExtent, GeographyUnit.DecimalDegree, map.Width, map.Height);
            DrawImage();
        }
    }

    private void Query_Btn_Click(object sender, EventArgs e)
    {
        string sql = textBox1.Text;
        featureLayer.FeatureSource.Open();
        DataTable dataTable = featureLayer.FeatureSource.ExecuteQuery(sql);

        dataGridView1.Rows.Clear();
        dataGridView1.Columns.Clear();

        foreach (DataColumn column in dataTable.Columns)
        {
            DataGridViewColumn dgColumn = new DataGridViewColumn();
            DataGridViewCell cell = new DataGridViewTextBoxCell();
            cell.Style.BackColor = Color.LightGray;
            dgColumn.CellTemplate = cell;
            dgColumn.HeaderText = column.ColumnName;
            dataGridView1.Columns.Add(dgColumn);
        }

        foreach (DataRow row in dataTable.Rows)
        {
            int index = dataGridView1.Rows.Add();
            for (int i = 0; i < dataTable.Columns.Count; i++)
            {
                Collection<object> objs = row[i] as Collection<object>;
                string objValues = "";
                foreach (object obj in objs)
                {
                    objValues += obj.ToString();
                }
                dataGridView1.Rows[index].Cells[i].Value = objValues;
            }
        }
        featureLayer.FeatureSource.Close();
    }
}